@@ -48,10 +48,21 @@ class Event:
 
     def __init__(self, kb, type, trigger, var, patterns, models):
         self.kb = kb
-        self.type = type
+
+        if type == Event.NEW_CLASS_INSTANCE:
+            self.type = Event.NEW_INSTANCE
+            self.var = "?instance"
+            self.patterns = [parse_stmt("?instance rdf:type %s" % klass[0]) for klass in patterns]
+        elif type == Event.NEW_CLASS_INSTANCE_ONE_SHOT:
+            self.type = Event.NEW_INSTANCE_ONE_SHOT
+            self.var = "?instance"
+            self.patterns = [parse_stmt("?instance rdf:type %s" % klass[0]) for klass in patterns]
+        else:
+            self.type = type
+            self.var = var
+            self.patterns = patterns
+
         self.trigger = trigger
-        self.var = var
-        self.patterns = patterns
         self.models = models
 
         self.id =  "evt_" + str(hash(self.type + \
@@ -83,12 +94,16 @@ class Event:
         if self.type in [Event.NEW_INSTANCE, Event.NEW_INSTANCE_ONE_SHOT]:
             instances = set(self.kb.store.query([self.var], self.patterns, frozenset(self.models)))
             newinstances = instances - self.previous_instances
-        
+
+            # previous_instances must be set to the current set of matching instance
+            # else we won't trigger events when an instance disappear and 
+            # re-appear later.
+            self.previous_instances = instances
+
             if not newinstances:
                 return False
 
             self.content = [i for i in newinstances] # for some reason, calling list() does not work
-            self.previous_instances = self.previous_instances | instances
             return True
 
 
@@ -541,7 +556,7 @@ class MinimalKB:
             res = None
             if args or kwargs:
                 res = f(*args, **kwargs)
-                if name=="subscribe":
+                if name in ["subscribe", "registerEvent"]:
                     self.eventsubscriptions.setdefault(res, []).append(client)
             else:
                 res = f()
